#define LIGHTS 2
#define TEXTURE_UNIT_SHADOWMAP_START 2
varying vec4 diffuse[LIGHTS], ambient[LIGHTS];
varying vec3 halfVector[LIGHTS];
varying float dist[LIGHTS];

varying vec3 ecDir;
varying vec4 ambientGlobal;
varying vec3 normal;

varying vec3 reflection;

uniform sampler2D tex;
uniform samplerCube cubeTex;
 
void main()
{
	float att;
	float NdotL, NdotHV;
	vec3 halfV;
	vec4 color = ambientGlobal;
	float shadow;
	
	vec3 n = normalize(normal);	
	
	for (int i = 0; i < LIGHTS; i++) {		
		vec3 lightDir = normalize(gl_LightSource[i].position.xyz - ecDir);
		
		float spotEffect = dot(
			normalize(gl_LightSource[i].spotDirection),
			normalize(-lightDir));
				
		if (spotEffect > gl_LightSource[i].spotCosCutoff) {
			
			NdotL = max(dot(n, lightDir), 0.0);

			if (NdotL > 0.0) {
			
				spotEffect = pow(spotEffect, gl_LightSource[i].spotExponent);
				att =
					spotEffect / (	gl_LightSource[i].constantAttenuation +
									gl_LightSource[i].linearAttenuation * dist[i] +
									gl_LightSource[i].quadraticAttenuation * dist[i] * dist[i]);

				color += att * (diffuse[i] * NdotL + ambient[i]);

				halfV = normalize(halfVector[i]);
				NdotHV = max(dot(n, halfV), 0.0);
				color +=	att * 
							gl_FrontMaterial.specular *
							gl_LightSource[i].specular *
							pow(NdotHV,gl_FrontMaterial.shininess);
			}
		}
	}
		
	vec4 texel = texture2D(tex, gl_TexCoord[0].st);	
	
	gl_FragColor =
		0.4 * textureCube(cubeTex, normalize(reflection)) +
		0.6 * vec4(color.rgb * texel.rgb, color.a * texel.a);
}