#include "HShadowMap.h"
#include <cmath>
 
HShader *HShadowMap::_shaderDepth = 0;
GLint HShadowMap::_textureSampler[MAX_SHADOWS];
 
/*! \brief ShadowMap Object
* \param texWidth Must be power of 2
* \param texHeight Must be power of 2 */
HShadowMap::HShadowMap(int texWidth, int texHeight) {
 
_texWidth = texWidth;
_texHeight = texHeight;
 
if (_shaderDepth == 0) {
_shaderDepth = new HShader("shadow");
glEnable(_TEXTURE_TYPE_DEPTH);
 
for (GLuint i = 0; i < MAX_SHADOWS; i++)
HShadowMap::_textureSampler[i] = i + _TEXTURE_UNIT_DEPTH - GL_TEXTURE0;
}
}
 
HShadowMap::~HShadowMap() {
glDeleteFramebuffersEXT(1, &_frameBufferDepth);
glDeleteTextures(1, &_textureDepth);
}
 
void HShadowMap::setViewMatrix(
GLdouble eyeX, GLdouble eyeY, GLdouble eyeZ,
GLdouble centerX, GLdouble centerY, GLdouble centerZ,
GLdouble upX, GLdouble upY, GLdouble upZ) {
 
float l = sqrt(upX*upX + upY*upY + upZ*upZ);
 
upX /= l;
upY /= l;
upZ /= l;
 
glMatrixMode(GL_MODELVIEW);
glPushMatrix();
glLoadIdentity();
gluLookAt(eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ);
glGetFloatv(GL_MODELVIEW_MATRIX, _viewMatrix);
glPopMatrix();
 
_viewDirection[0] = centerX - eyeX;
_viewDirection[1] = centerY - eyeY;
_viewDirection[2] = centerZ - eyeZ;
 
l = sqrt(
_viewDirection[0]*_viewDirection[0] +
_viewDirection[1]*_viewDirection[1] +
_viewDirection[2]*_viewDirection[2]);
 
_viewDirection[0] /= l;
_viewDirection[1] /= l;
_viewDirection[2] /= l;
_viewUpVector[0] = upX;
_viewUpVector[1] = upY;
_viewUpVector[2] = upZ;
}
 
void HShadowMap::setLightProjectionMatrix(GLdouble fovy, GLdouble zNear, GLdouble zFar) {
 
glMatrixMode(GL_PROJECTION);
glPushMatrix();
glLoadIdentity();
gluPerspective(fovy, (float)_texWidth / (float)_texHeight, zNear, zFar);
glGetFloatv(GL_PROJECTION_MATRIX, _lightProjectionMatrix);
glPopMatrix();
glMatrixMode(GL_MODELVIEW);
}
 
 
void HShadowMap::loadViewMatrix() {
glLoadMatrixf(_viewMatrix);
glActiveTexture(GL_TEXTURE0);
glMatrixMode(GL_TEXTURE);
glLoadMatrixf(_viewMatrix);
glMatrixMode(GL_MODELVIEW);
}
 
void HShadowMap::activateDepthTex() {
// Depth Texture
glActiveTexture(_TEXTURE_UNIT_DEPTH);
}
 
void HShadowMap::bindDepthTexture(GLuint texUnit) {
glActiveTexture(texUnit);
glBindTexture(_TEXTURE_TYPE_DEPTH, _textureDepth);
}
 
void HShadowMap::bindDepthTexture() {
glActiveTexture(_TEXTURE_UNIT_DEPTH);
glBindTexture(_TEXTURE_TYPE_DEPTH, _textureDepth);
}
 
void HShadowMap::unbindDepthTexture() {
glActiveTexture(_TEXTURE_UNIT_DEPTH);
glBindTexture(_TEXTURE_TYPE_DEPTH, 0);
}
 
void HShadowMap::setupDepthTex() {
glTexParameteri(_TEXTURE_TYPE_DEPTH, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
glTexParameteri(_TEXTURE_TYPE_DEPTH, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
glTexParameteri(_TEXTURE_TYPE_DEPTH, GL_TEXTURE_WRAP_S, GL_CLAMP);
glTexParameteri(_TEXTURE_TYPE_DEPTH, GL_TEXTURE_WRAP_T, GL_CLAMP);
glTexParameteri(_TEXTURE_TYPE_DEPTH, GL_DEPTH_TEXTURE_MODE, GL_INTENSITY);
}
