#ifndef HShadowMap_h
#define HShadowmap_h

#include "platform.h"
#include "HShader.h"
#include "light/HSpotLight.h"

using namespace std;

class HShadowMap {
public:
	HShadowMap(int width, int height);
	~HShadowMap();

	void activateTex();
	void setupTex();

	void setViewMatrix(
		GLdouble eyeX, GLdouble eyeY, GLdouble eyeZ,
		GLdouble centerX, GLdouble centerY, GLdouble centerZ,
		GLdouble upX, GLdouble upY, GLdouble upZ
	);

	void setLightViewMatrix();

	void setLightProjectionMatrix(GLdouble fovy, GLdouble zNear, GLdouble zFar);

	void setTextureMatrixToLightSpaceTransform();

	void loadViewMatrix();
	void assignLight(HSpotLight *spotLight);

	void beginRenderDepth();
	void endRenderDepth();

	void beginRenderPass();
	void endRenderPass();

	void bindTextureToShader(string varName, HShader &shader);
	void bindTexture();
	void unbindTexture();

	float _upX, _upY, _upZ;

	const GLuint _TEXTURE_TYPE;
	const GLuint _TEXTURE_UNIT;

private:
	HSpotLight *_spotLight;
	int _texWidth, _texHeight;
	GLuint _texture, _frameBuffer;

	GLfloat _viewMatrix[16];
	GLfloat _lightViewMatrix[16];
	GLfloat _lightProjectionMatrix[16];

	GLfloat _viewDirection[3];
	GLfloat _viewUpVector[3];
	
	static HShader *_shader;
};

#endif